<?php
defined('BASEPATH') OR exit('No direct script access allowed');

/**
 * 自定义配置表
 * Class Setting_model
 */
class Setting_model extends MY_Model {

    public $table = 'setting';

    public function __construct() {
        parent::__construct();
    }

    /**
     * 验证规则
     *
     * 写入配置到数据库时要注意：
     * -配置key不能重名
     * -配置key不能与group重名
     *
     * @return array
     */
    public function rules()
    {
        return [
            'site_name' => ['rules'=>'required'],
            'base_url'  => ['rules'=>'required'],
        ];
    }

    /**
     * 获取配置分组列表（缓存）
     * @param bool $cache
     * @return array
     */
    public function get_group($cache = TRUE)
    {
        $cache_key = 'setting';
        $data = $this->cache->get($cache_key);
        if ( ! $cache || ! $data)
        {
            $list = $this->db->distinct()->select('group')->get($this->table)->result_array();

            $data = [];
            if ( ! empty($list)) {
                foreach ($list as $v) {
                    $data[] = $v['group'];
                }
            }

            if ($cache) {
                $this->cache->save($cache_key, $data);
            }
        }
        return $data;
    }

    /**
     * 获取配置列表（按分组）（缓存）
     * @param string $group
     * @param bool $cache
     * @return array|null
     */
    public function get_setting($group = '', $cache = TRUE)
    {
        $groups = $this->get_group($cache);
        if ( ! in_array($group, $groups)) {
            $group = 'all';
        }

        $cache_key = 'setting_'.$group;
        $data = $this->cache->get($cache_key);
        if ( ! $cache || ! $data)
        {
            if($group != 'all'){
                $this->db->where('group', $group);
            }
            $list = $this->db->get($this->table)->result_array();

            $data = [];
            if ( ! empty($list)) {
                foreach ($list as $v) {
                    $data[$v['key']] = $v['value'];
                }
            }

            if ($cache) {
                $this->cache->save($cache_key, $data);
            }
        }
        return $data;
    }

    public function get_contact_fields($type="reg_bind",$cache = true){
        $cache_key = 'contact_'.$type;
        $data = $this->cache->get($cache_key);
        if ( ! $cache || ! $data){
                $contact_fields = json_decode(get_config_item('contact_fields','','[]'),true);
                if(empty($contact_fields)){
                    $contact_fields[] = ['name'=>'QQ','field'=>'qq','status'=>1,'bind'=>get_config_item('reg_bind_qq')];
                    $contact_fields[] = ['name'=>'SKYPE','field'=>'skype','status'=>1,'bind'=>0];
                    $contact_fields[] = ['name'=>'Telegram','field'=>'telegram','status'=>0,'bind'=>0];
                    $contact_fields[] = ['name'=>'Whatsapp','field'=>'whatsapp','status'=>0,'bind'=>0];
                }
                foreach ($contact_fields as $key =>$val){
                    $contact_fields[$key]['class'] = in_array($val['field'],['telegram','qq','weixin','whatsapp','skype','weibo','facebook','tiktok','twitter']) ? $val['field'] : 'qita';
                }
                // 注册绑定
                $data = array_filter($contact_fields,function ($row) use ($type){
                    //$row['class'] = in_array($row['field'],['telegram','qq','weixin','whatsapp','skype','weibo','facebook','tiktok','twitter']) ? $row['field'] : 'qita';
                    if($type=="reg_bind" && $row['bind']){
                        return $row;
                    }elseif($type=="info" && $row['status']){
                        return $row;
                    }elseif($type=="admin"){
                        return $row;
                    }
                });
            if ($cache) {
                $this->cache->save($cache_key, $data);
            }
        }
        return $data;
    }

    /**
     * 设置一个配置项
     * @param $key
     * @param $value
     * @param string $group
     */
    public function set_value($key, $value, $group='config'){
        if($this->db->get_where($this->table, ['key'=>$key], 1)->row_array()){
            // 更新配置
            $this->db->update($this->table, ['value'=>$value], ['key'=>$key]);
        }else{
            // 添加配置
            $this->db->insert($this->table, ['key'=>$key,'value'=>$value,'group'=>$group]);
        }
        $this->common_service->clear_cache();
    }

    /**
     * 动态获取一个配置值
     * @param $key
     * @return mixed
     */
    public function get_value($key){
        $this->db->select('value');
        $this->db->where('key', $key);
        $row = $this->db->get($this->table, 1)->row_array();
        return element('value', $row);
    }

    // 后台登录地址随机码
    public function backend_rand($type='', $rand=''){
        $file = CACHEPATH.'backend_rand.txt';
        if($type == 'set'){
            // 保存随机码
            $this->load->helper('file');
            write_file($file, trim($rand));
        }else{
            // 读取随机码
            if(!is_file($file)){ return ''; }
            $rand = file_get_contents($file);
            return trim($rand);
        }
    }

    // 格式化返回到期通知天数
    public function get_server_end_days(){
        $server_end_days = trim(get_config_item('server_end_days','server'));
        $server_end_days = $server_end_days ? trim($server_end_days) : '1 3 7';
        if($server_end_days == -1){ // 不通知
            return false;
        }
        $server_end_days = str_split(str_replace(" ","",$server_end_days),1);
        return $server_end_days;
    }

    // 清除日志时间段
    public function get_clear_log_times($key = '', $field = '', $type = ''){
        $config = [
            'one_week_before' => ['name'=>_('一星期前'),'time'=>strtotime('-1 week')],
            'one_month_before' => ['name'=>_('一个月前'),'time'=>strtotime('-1 month')],
            'three_month_before' => ['name'=>_('三个月前'),'time'=>strtotime('-3 month')],
            'six_month_before' => ['name'=>_('半年前'),'time'=>strtotime('-6 month')],
            'this_year_before' => ['name'=>_('今年以前'),'time'=>strtotime(mdate('%Y0101 00:00:00'))],
            'all' => ['name'=>_('所有时间'),'time'=>now(),'line'=>true],
        ];
        return $this->config->get_self_config($config,$key,$field,$type);
    }

    // 实名认证接口
    public function real_api_config($key = '', $field = '', $type = '', $hide_check=false,$input=[]){
        $config = [
            '0' => ['name'=>_('后台审核')],
            '5' => ['name'=>_('比利联实名')],
        ];
        $config[5] = ['name'=>get_config_item('real_bll_channel','api','alipay')=='alipay' ? _('实名认证(支付宝)') : _('实名认证(E政通)')];
        if(has_addons('aliapiReal')){
            $config[1] = ['name'=>_('天眼数聚').'（'._('阿里云市场').'）','company'=>1];
        }
        if(has_addons('alipayReal')){
            $config[2] = ['name'=>_('支付宝')];
        }
        if(has_addons('tencentface')){
            $config[3] = ['name'=>_('腾讯云人脸核身')];
        }
        if(has_addons('tencentface')){
            $config[3] = ['name'=>_('腾讯云人脸核身')];
        }
        if(has_addons('tencentfaceEid')){
            $config[4] = ['name'=>_('腾讯云人脸E证通')];
        }
        if($hide_check === true){ // 不需要后台审核1122
            unset($config[0]);
        }elseif($hide_check === 'enable'){ // 不显示未启用的
            unset($config[0]);
            if($cert_type = get_config_item('cert_type','api')){
                $config = [$cert_type=>$config[$cert_type]];
            }else{
                if(!get_config_item('cert_type_1','api')){ unset($config[1]); }
                if(!get_config_item('cert_type_2','api')){ unset($config[2]); }
                if(!get_config_item('cert_type_3','api')){ unset($config[3]); }
                if(!get_config_item('cert_type_4','api')){ unset($config[4]); }
                if(!get_config_item('cert_type_5','api')){ unset($config[5]); }
            }
        }
        if(element('type',$input) == 2){
            // 支持企业认证的接口
            foreach($config as $k=>$v){
                if(empty(element('company',$v))){
                    unset($config[$k]);
                }
            }
        }
        return $this->config->get_self_config($config,$key,$field,$type);
    }

    // 短信接口
    public function sms_api_config($key = '', $field = '', $type = ''){
        $config = [
            '4' => ['name'=>_('比利联短信'),'key'=>'bllsms'],
            '100' => ['name'=>_('短信屋短信'),'key'=>'quansms'],
        ];
        if(has_addons('aliyunsms')){
            $config[1] = ['name'=>_('阿里大于'),'key'=>'aliyunsms'];
        }
        if(has_addons('baosms')){
            $config[2] = ['name'=>_('短信宝'),'key'=>'baosms'];
        }
        if(has_addons('tencentsms')){
            $config[3] = ['name'=>_('腾讯云短信'),'key'=>'tencentsms'];
        }
        return $this->config->get_self_config($config,$key,$field,$type);
    }

    // 邮箱接口
    public function mail_api_config($key = '', $field = '', $type = ''){
        $config = [
            '1' => ['name'=>'SMTP'],
            '2' => ['name'=>'Send Cloud'],
        ];
        return $this->config->get_self_config($config,$key,$field,$type);
    }

    /**
     * 支持的语言包类型
     * @param string $key
     * @param string $field
     * @param string $type
     */
    public function get_language_config($key = '', $field = '', $type = '', $input = []){
        $config = array(
            /**
             * support=(all=支持所有客户端 home=支持前台 admin=支持后台，多个用逗号分隔)
             * lang=编辑器语言包 locale=站点语言包
             */
            'chinese' => ['key'=>'chinese','name'=>'简体中文','lang'=>'zh-cn','class'=>'flag-CN','support'=>'all','locale'=>'zh_CN'],
            'chinesehk' => ['key'=>'chinese','name'=>'繁体中文','lang'=>'zh-hk','class'=>'flag-HK','support'=>'all','locale'=>'zh_HK'],
            'english' => ['key'=>'english','name'=>'English','lang'=>'en','class'=>'flag-GB','support'=>'all','locale'=>'en_US'],
        );
        $support = element('support',$input,'');
        if($support){
            foreach ($config as $k=>$v){
                $supports = explode(',',element('support',$v));
                if(in_array('all', $supports)){ continue; }
                if(! in_array($support, $supports)){
                    unset($config[$k]);
                }
            }
        }
        return $this->config->get_self_config($config,$key,$field,$type);
    }

    /**
     * 支持的货币类型
     * @param string $key
     * @param string $field
     * @param string $type
     */
    public function get_currency_config($key = '', $field = '', $type = ''){
        $config = array(
            'CNY' => ['name'=>_('人民币'),'currency'=>'￥','money'=>'元'],
            'HKD' => ['name'=>_('港币'),'currency'=>'HK$','money'=>'HKD'],
            'TWD' => ['name'=>_('新台币'),'currency'=>'NT$','money'=>'NTD'],
            'USD' => ['name'=>_('美元'),'currency'=>'$','money'=>'USD'],
            'EUR' => ['name'=>_('欧元'),'currency'=>'€','money'=>'EUR'],
            'GBP' => ['name'=>_('英镑'),'currency'=>'￡','money'=>'GBP'],
            'JPY' => ['name'=>_('日元'),'currency'=>'JPY￥','money'=>'JPY'],
            '' => ['name'=>_('其他货币'),'currency'=>'','money'=>''],
        );
        return $this->config->get_self_config($config,$key,$field,$type);
    }

    // 检测调试模式
    public function check_debug_mod(){
        // 检查调试模式是否启用
        if(! get_config_item('env_debug','env')){
            return response(['code'=>1,'msg'=>_('未启用调试模式！')]);
        }
        return true;
    }

    /**
     * 设置分页
     * @param $size
     * @param $input
     */
    public function set_page_size($size=0, $input=[]){
        // 分页数量记忆模式
        $this->load->helper('cookie');
        $backend_size_mod = get_config_item('backend_size_mod','admin',0);
        $routermd5 = element('routermd5', $input, '');

        // 获取当前路由分页显示数
        if($size == 0){
            $page_size = 0;
            if($backend_size_mod == 1){ // 按路由（临时）
                $cookie_key = 'router_curr_pagesize';
                $record = get_cookie($cookie_key);
                if($record){
                    $record = json_decode($record,true);
                    if($routermd5 == element('routermd5', $record)){
                        $page_size = element('size', $record);
                    }
                }
            }elseif($backend_size_mod == 2){ // 按路由（记忆）
                $cookie_key = 'router_list_pagesize';
                $record = get_cookie($cookie_key);
                if($record){
                    $record = json_decode($record,true);
                    if(element($routermd5, $record)){
                        $page_size = element($routermd5, $record);
                    }
                }
            }
            if($page_size){
                $this->config->set_config('backend_page_size',$page_size,'admin');
            }
            return;
        }

        // 手动设置路由显示数
        if($backend_size_mod == 1){ // 按路由（临时）
            $cookie_key = 'router_curr_pagesize';
            $record = json_encode(['routermd5'=>$routermd5,'size'=>$size]);
            set_cookie($cookie_key, $record, 365*24*3600);
        }elseif($backend_size_mod == 2){ // 按路由（记忆）
            $cookie_key = 'router_list_pagesize';
            $record = get_cookie($cookie_key);
            if($record){ $record = json_decode($record,true); }
            else{ $record = []; }
            $record[$routermd5] = $size;
            set_cookie($cookie_key, json_encode($record), 365*24*3600);
        }elseif($backend_size_mod == 3){ // 全局记忆
            $this->set_value('backend_page_size', $size);
        }
    }

}

?>